package tesuji.games.go;

import tesuji.games.go.GoArray;

/**
 	This is an important class that is used to mark points that
 	have already been processed. Each time the method getNewMarker
 	is called, the information is reset and all points are not marked.

 	<br><br>Creation date: (17-May-01 2:50:11 PM)<br><br>

	Product: Tesuji Software Go Library.<br><br>

	<font color="#CC6600"><font size=-1>
	Copyright (c) 2001 Tesuji Software B.V.<br>
	All rights reserved.<br><br>

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.<br><br>

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
	COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
	IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
	CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.<br><br>

	Except as contained in this notice, the name of a Tesuji Software
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of Tesuji Software.<br><br>
	<font color="#00000"><font size=+1>

	@author: Mark Boon
*/
public class BoardMarker
{
	private int labelNr;
	private int[] markers;

	public BoardMarker()
	{
		markers = GoArray.createIntegers();
		labelNr = 1;
	}

	/**
	 	This causes all points previously marked as 'set'
	 	to be unmarked.
	
	 	<br><br>Creation date: (17-May-01 2:52:16 PM)<br><br>
	*/
	public final void getNewMarker()
	{
		labelNr++;
		if (labelNr==0)
		{
			labelNr = 1;
			GoArray.clear(markers);
		}
	}

	/**
	 	Mark a point as set.
	
	 	<br><br>Creation date: (17-May-01 2:55:38 PM)<br><br>
	
	 	@param: int xy is the coordinate of the point to mark as set.
	*/
	public final void set( int xy )
	{
		markers[xy] = labelNr;
	}

	/**
	 	Test whether a point was already marked or not.
	
	 	<br><br>Creation date: (17-May-01 2:53:28 PM)<br><br>
	
	 	@param: int xy coordinate of the point to test
	 	@return: boolean whether the point was malready marked or not.
	*/
	public final boolean notSet( int xy )
	{
		return markers[xy]!=labelNr;
	}

	/**
	 	Test whether a point was already marked or not.

	 	<br><br>Creation date: (17-May-01 2:53:28 PM)<br><br>

	 	@param: int xy coordinate of the point to test
	 	@return: boolean whether the point was already marked or not.
	*/
	public final boolean isSet( int xy )
	{
		return markers[xy]==labelNr;
	}
}