package tesuji.games.go;

import java.util.Stack;

/**
 	This class defines arrays used for a Go-board.
 	All coordinates are coded such that x,y translates to x+y*20.
	There are methods to create and clear arrays of the appropriate size
	and a lot of utility methods to manipulate coordinates.

 	<br><br>Creation date: (17-May-01 3:07:20 PM)<br><br>

	Product: Tesuji Software Go Library.<br><br>

	<font color="#CC6600"><font size=-1>
	Copyright (c) 2001 Tesuji Software B.V.<br>
	All rights reserved.<br><br>

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.<br><br>

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
	COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
	IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
	CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.<br><br>

	Except as contained in this notice, the name of a Tesuji Software
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of Tesuji Software.<br><br>
	<font color="#00000"><font size=+1>

	@author: Mark Boon
*/
public class GoArray
{
	public static final int WIDTH = 20;
	public static final int FIRST = 21;
	public static final int MAX = 422;
	public static final int LAST = 400;
	public static final int ILLEGAL_VALUE = Integer.MIN_VALUE;

	// Since Java has no facility to clear arrays fast, these
	// arrays are kept empty and copied over existing arrays
	// when they need to be cleared.
	private static final byte[] byteZeros =		new byte[MAX];
	private static final short[] shortZeros =	new short[MAX];
	private static final int[] intZeros =		new int[MAX];
	private static final long[] longZeros =		new long[MAX];
	private static final Object[] objectNulls =	new Object[MAX];

	private static IntStack toDoList =			new IntStack();
	private static BoardMarker marker =			new BoardMarker();

	private static byte[][] rowArrays =			new byte[WIDTH][0];

	private static Stack byteArrayStack;
	private static Stack intArrayStack;

	static
	{
		for (int i=0; i<MAX; i++) byteZeros[i] = 0;
		for (int i=0; i<MAX; i++) shortZeros[i] = 0;
		for (int i=0; i<MAX; i++) intZeros[i] = 0;
		for (int i=0; i<MAX; i++) longZeros[i] = 0;
		for (int i=0; i<MAX; i++) objectNulls[i] = null;
	}

	public static final void
	clear(byte[] array)
	{
		System.arraycopy(byteZeros,0,array,0,MAX);
	}
    
	public static final void
	clear(short[] array)
	{
		System.arraycopy(shortZeros,0,array,0,MAX);
	}
    
	public static final void clear(int[] array)
	{
		System.arraycopy(intZeros,0,array,0,MAX);
	}
    
	public static final void
	clear(long[] array)
	{
		System.arraycopy(longZeros,0,array,0,MAX);
	}
    
	public static final void
	clear(Object[] array)
	{
		System.arraycopy(objectNulls,0,array,0,MAX);
	}
    
	public static final void
	copy(byte[] src, byte[] dest)
	{
		System.arraycopy(src,0,dest,0,MAX);
	}
    
	public static final void
	copy(short[] src, short[] dest)
	{
		System.arraycopy(src,0,dest,0,MAX);
	}
    
	public static final void
	copy(int[] src, int[] dest)
	{
		System.arraycopy(src,0,dest,0,MAX);
	}
    
	public static final void
	copy(long[] src, long[] dest)
	{
		System.arraycopy(src,0,dest,0,MAX);
	} 
	
	public static final void
	copy(Object[] src, Object[] dest)
	{
		System.arraycopy(src,0,dest,0,MAX);
	} 
	
	public static final byte[]
	createBytes()
	{
		if (byteArrayStack==null)
			byteArrayStack = new Stack();

		if (byteArrayStack.isEmpty())
			return new byte[MAX];
		else
			return (byte[]) byteArrayStack.pop();
	}

	public static final short[]
	createShorts()
	{
		return new short[MAX];
	}

	public static final int[]
	createIntegers()
	{
		if (intArrayStack==null)
			intArrayStack = new Stack();

		if (intArrayStack.isEmpty())
			return new int[MAX];
		else
			return (int[]) intArrayStack.pop();
	}

	public static final long[]
	createLongs()
	{
		return new long[MAX];
	}
	
	public static final Object[]
	createObjects()
	{
		return new Object[MAX];
	}
	
	public static final void
	link(int value, int startXY, byte[] board, int[] numbers)
	{
		byte color = board[startXY];
		toDoList.clear();
		toDoList.push(startXY);
		do
		{
			int xy = toDoList.pop();
			numbers[xy] = value;
			int left = left(xy);
			if (numbers[left]!=value && board[left]==color)
				toDoList.push(left);
			int right = right(xy);
			if (numbers[right]!=value && board[right]==color)
				toDoList.push(right);
			int above = above(xy);
			if (numbers[above]!=value && board[above]==color)
				toDoList.push(above);
			int below = below(xy);
			if (numbers[below]!=value && board[below]==color)
				toDoList.push(below);
		}
		while (!toDoList.isEmpty());
	}
	
	public static final void
	link(byte value, int startXY, byte[] board, byte[] numbers)
	{
		byte color = board[startXY];
		toDoList.clear();
		toDoList.push(startXY);
		do
		{
			int xy = toDoList.pop();
			numbers[xy] = value;
			int left = left(xy);
			if (numbers[left]!=value && board[left]==color)
				toDoList.push(left);
			int right = right(xy);
			if (numbers[right]!=value && board[right]==color)
				toDoList.push(right);
			int above = above(xy);
			if (numbers[above]!=value && board[above]==color)
				toDoList.push(above);
			int below = below(xy);
			if (numbers[below]!=value && board[below]==color)
				toDoList.push(below);
		}
		while (!toDoList.isEmpty());
	}
	
	public static final void
	link(Object value, int startXY, byte[] board, Object[] objects)
	{
		byte color = board[startXY];
		toDoList.clear();
		toDoList.push(startXY);
		do
		{
			int xy = toDoList.pop();
			objects[xy] = value;
			int left = left(xy);
			if (objects[left]!=value && board[left]==color)
				toDoList.push(left);
			int right = right(xy);
			if (objects[right]!=value && board[right]==color)
				toDoList.push(right);
			int above = above(xy);
			if (objects[above]!=value && board[above]==color)
				toDoList.push(above);
			int below = below(xy);
			if (objects[below]!=value && board[below]==color)
				toDoList.push(below);
		}
		while (!toDoList.isEmpty());
	}

	/**
	 	Return the coordinate one point above the coordinate
	 	passed as a parameter.
	
	 	<br><br>Creation date: (17-May-01 3:14:46 PM)<br><br>
	*/
	public static final int above(int xy)
	{
		return xy-WIDTH;
	}

	/**
	 	Return the coordinate one point below the coordinate
	 	passed as a parameter.
	
	 	<br><br>Creation date: (17-May-01 3:14:46 PM)<br><br>
	*/
	public static final int below(int xy)
	{
		return xy+WIDTH;
	}

	public static final void checkRange(int xy)
	{
//		Assert.assert(xy>=FIRST && xy<=LAST);
	}

	/**
	 	This method creates an array for a Go board of a given size
	 	with an edge around the board-points. The size of the array
	 	is always the same, but the points not used are marked
	 	with the value GoConst.SIDE.
	
	 	<br><br>Creation date: (17-May-01 11:53:42 AM)<br><br>
	
	 	@param: int size is the board-size used. Maximum value is 19
	 	@return: byte[]
	*/
	public static final byte[] getBoardArray(int size)
	{
		byte[] board = createBytes();

		for (int i=0; i<MAX; i++)
		{
			int x = getX(i);
			int y = getY(i);

			if (x==0 || y==0 || x>size || y>size)
				board[i] = GoConst.SIDE;
			else
				board[i] = GoConst.EMPTY;
		}
		return board;
	}

	/**
	 	This method creates an array for a Go board of a given size
	 	with at each point the distance to the edge of the board.
	 	Any point that is not on the board gets a value 0.<br><br>

	 	Very sneakily, the board-size is stored in the last element
	 	of the array, so be careful...<br><br>

	 	This is basically an array with static data that is very handy.
	 	To minimize memory use, the array is reused for each board-size
	 	so make sure the array never gets used to store other data.
	
	 	<br><br>Creation date: (17-May-01 11:53:42 AM)<br><br>
	
	 	@param: int size is the board-size used. Maximum value is 19
	 	@return: byte[]
	*/
	public static final byte[] getRowArray(int size)
	{
		if (rowArrays[size].length!=0)
			return rowArrays[size];

		byte[] rows = createBytes();

		for (int i=0; i<MAX; i++)
		{
			int x = getX(i);
			int y = getY(i);

			int horizontal = Math.min(x,size+1-x);
			int vertical = Math.min(y,size+1-y);
			int row = Math.min(horizontal,vertical);
		
			if (row<=0)
				rows[i] = 0;
			else
				rows[i] = (byte) row;
		}
		
		rows[rows.length-1] = (byte) size;

		rowArrays[size] = rows;
		return rows;
	}

	public static final int getX(int xy)
	{
		return xy%WIDTH;
	}

	public static final int getY(int xy)
	{
		return xy/WIDTH;
	}

	/**
	 	<br><br>Creation date: (07-May-01 2:36:04 PM)<br><br>
	
	 	@param: int xy1
	 	@param: int xy2
	 	@return: boolean value whether xy1 and xy2 are next to each other..
	*/
	public static final boolean isNeighbour(int xy1, int xy2)
	{
		int xy3=xy1-xy2;
		if (xy3==1 || xy3==-1 || xy3==WIDTH || xy3==-WIDTH)
			return true;
		return false;
	}

	/**
	 	Return the coordinate one point to the left of the coordinate
	 	passed as a parameter.
	
	 	<br><br>Creation date: (17-May-01 3:14:46 PM)<br><br>
	*/
	public static final int left(int xy)
	{
		return xy-1;
	}

	/**
	 	Return the coordinate one point to the rigth of the coordinate
	 	passed as a parameter.
	
	 	<br><br>Creation date: (17-May-01 3:14:46 PM)<br><br>
	*/
	public static final int right(int xy)
	{
		return xy+1;
	}

	/**
		Convert a 2-dimensional coordinate to a 1 dimensional coordinate.
	
	 	<br><br>Creation date: (17-May-01 3:16:26 PM)<br><br>
	*/
	public static final int toXY(int x, int y)
	{
		return x+y*WIDTH;
	}

	public static final void recycleByteArray(byte[] array)
	{
		byteArrayStack.push(array);
	}

	public static final void recycleIntArray(int[] array)
	{
		intArrayStack.push(array);
	}
}