package tesuji.games.go;

import java.util.Stack;

import tesuji.games.go.GoArray;
import tesuji.games.general.Cursor;
import tesuji.games.general.FlyWeight;

/**
 	This class acts just like IntStack, except that it won't
 	allow the same value in the list more than once.
 	The restriction is that the values must be within the range
 	of 0 and GoArray.MAX.<br><br>

 	Typically this class is used to store (small) lists of
 	coordinates on a go-board.

 	<br><br>Creation date: (17-May-01 2:26:28 PM)<br><br>

	Product: Tesuji Software Go Library.<br><br>

	<font color="#CC6600"><font size=-1>
	Copyright (c) 2001 Tesuji Software B.V.<br>
	All rights reserved.<br><br>

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.<br><br>

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
	COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
	IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
	CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.<br><br>

	Except as contained in this notice, the name of a Tesuji Software
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of Tesuji Software.<br><br>
	<font color="#00000"><font size=+1>

	@author: Mark Boon
*/
public class UniqueList
	implements FlyWeight
{
	// Maximum number of items in the list.
	private static final int MAX = 361;
	
	// Here we record what items are already in the list.
	private byte[] bitmap;
	// This is an experiment. Any item for which the mask is 0 are not allowed.
	private static byte[] mask = GoArray.getRowArray(19);
	// Most calls delegate to this list.
	private IntStack list;

	 // This is the stack that owns the object when it gets
	 // created and to which it gets recycled after use.
	private Stack owner;

	public UniqueList()
	{
		list = new IntStack(MAX);
		bitmap = GoArray.createBytes();
	}

	public static void setMask(byte[] theMask)
	{
		mask = theMask;
	}

	public boolean
	add(int xy)
	{
		if (bitmap[xy]!=0 || mask[xy]==0)
			return false;

		list.push(xy);
		bitmap[xy] = 1;
		return true;
	}

	public boolean hasMember(int xy)
	{
		return (bitmap[xy]!=0);
	}

	public boolean
	isEmpty()
	{
		return list.isEmpty();
	}

	public int getSize()
	{
		return list.getSize();
	}

	public Cursor getIterator()
	{
		return list.getIterator();
	}

	public UniqueList(Stack ownerStack)
	{
		this();
		owner = ownerStack;
	}

	public void add(IntStack otherStack)
	{
		for (int i=otherStack.getSize()-1; i>=0; i--)
			add(otherStack.get(i));
	}

	public void clear()
	{
		list.clear();
		GoArray.clear(bitmap);
	}

	public int get(int index)
	{
		return list.get(index);
	}

	public final void recycle()
	{
		if (owner!=null)
			owner.push(this);
	}

	public void remove(int xy)
	{
		list.remove(xy);
		bitmap[xy] = 0;
	}

	// Setting the size of the number of items in the list.
	// Should only be used if the current size is greater than
	// the new size and can be used to 'pop' more than one item.
	// Use with care.
	public void setSize(int size)
	{
		while (list.getSize()>size)
		{
			int xy = list.pop();
			bitmap[xy] = 0;
		}
	}
}