package tesuji.games.go;

import java.util.Stack;

import tesuji.games.general.Cursor;
import tesuji.games.general.Point;

/**
 	Here are some routines that are generally usefull and
 	convenient to have.

 	<br><br>Creation date: (17-May-01 2:36:43 PM)<br><br>

	Product: Tesuji Software Go Library.<br><br>

	<font color="#CC6600"><font size=-1>
	Copyright (c) 2001 Tesuji Software B.V.<br>
	All rights reserved.<br><br>

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.<br><br>

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
	COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
	IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
	CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.<br><br>

	Except as contained in this notice, the name of a Tesuji Software
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of Tesuji Software.<br><br>
	<font color="#00000"><font size=+1>

	@author: Mark Boon
*/
public class Util
{
	private static Stack intStackStack = new Stack();
	private static BoardMarker labels = new BoardMarker();
	private static Stack pointStack = new Stack();
	private static IntStack toDoList = new IntStack();
	private static Stack uniqueListStack = new Stack();

	public static String printCoordinate(int xy)
	{
		int x = xy%GoArray.WIDTH;
		int y = xy/GoArray.WIDTH;
		if (x<10)
			return " "+x+","+y;
		else
			return ""+x+","+y;
	}

	/**
	 	Allocate an IntStack of default size. Instead of allocating
	 	new objects each time, the objects created through this method
	 	can be 'recycled' and then reused. This is much more efficient
	 	than relying on the VM to allocate an object and then later
	 	garbage-collect it again.

	 	<br><br>Creation date: (17-May-01 2:37:36 PM)<br><br>
	*/
	public static IntStack getIntStack()
	{
		if (intStackStack.isEmpty())
			return new IntStack(intStackStack);

		IntStack intStack = (IntStack) intStackStack.pop();
		intStack.clear();
		return intStack;
	}

	/**
	 	Same as the other getNlib method except it doesn't store the liberties.

	 	<br><br>Creation date: (19-May-01 6:52:56 PM)<br><br>
	*/
	public static final int getNlib(int startXY, int max, byte color, byte[] board)
	{
		if (board[startXY]==GoConst.SIDE)
			return 0;

		labels.getNewMarker();

		int nLibs = 0;

		toDoList.clear();
		toDoList.push(startXY);
		do
		{
			int xy = toDoList.pop();
			labels.set(xy);
			int left = GoArray.left(xy);
			if (board[left]==GoConst.EMPTY && labels.notSet(left))
			{
				if (++nLibs>=max)
					return nLibs;
				labels.set(left);
			}
			int right = GoArray.right(xy);
			if (board[right]==GoConst.EMPTY && labels.notSet(right))
			{
				if (++nLibs>=max)
					return nLibs;
				labels.set(right);
			}
			int above = GoArray.above(xy);
			if (board[above]==GoConst.EMPTY && labels.notSet(above))
			{
				if (++nLibs>=max)
					return nLibs;
				labels.set(above);
			}
			int below = GoArray.below(xy);
			if (board[below]==GoConst.EMPTY && labels.notSet(below))
			{
				if (++nLibs>=max)
					return nLibs;
				labels.set(below);
			}

			if (board[left]==color && labels.notSet(left))
				toDoList.push(left);
			if (board[right]==color && labels.notSet(right))
				toDoList.push(right);
			if (board[above]==color && labels.notSet(above))
				toDoList.push(above);
			if (board[below]==color && labels.notSet(below))
				toDoList.push(below);
		}
		while (!toDoList.isEmpty());

		return nLibs;
	}

	/**
	 	Get the number of liberties of a chain. The stones are in the 'board'
		array that is passed in. The liberties are put on the stack parameter.
		Only a maximum number of liberties are retrieved, depending on the 'max'
		parameter. If the starting point is empty, then the liberties
		are counted as if a stone of 'color' was played there.

	 	<br><br>Creation date: (19-May-01 6:52:56 PM)<br><br>
	*/
	public static final int getNlib(int startXY, int max, byte color, byte[] board, IntStack libertyStack)
	{
		if (board[startXY]==GoConst.SIDE)
			return 0;

		labels.getNewMarker();
		libertyStack.clear();

		int nLibs = 0;

		toDoList.clear();
		toDoList.push(startXY);
		do
		{
			int xy = toDoList.pop();
			labels.set(xy);
			int left = GoArray.left(xy);
			if (board[left]==GoConst.EMPTY && labels.notSet(left))
			{
				libertyStack.push(left);
				if (++nLibs>=max)
					return nLibs;
				labels.set(left);
			}
			int right = GoArray.right(xy);
			if (board[right]==GoConst.EMPTY && labels.notSet(right))
			{
				libertyStack.push(right);
				if (++nLibs>=max)
					return nLibs;
				labels.set(right);
			}
			int above = GoArray.above(xy);
			if (board[above]==GoConst.EMPTY && labels.notSet(above))
			{
				libertyStack.push(above);
				if (++nLibs>=max)
					return nLibs;
				labels.set(above);
			}
			int below = GoArray.below(xy);
			if (board[below]==GoConst.EMPTY && labels.notSet(below))
			{
				libertyStack.push(below);
				if (++nLibs>=max)
					return nLibs;
				labels.set(below);
			}

			if (board[left]==color && labels.notSet(left))
				toDoList.push(left);
			if (board[right]==color && labels.notSet(right))
				toDoList.push(right);
			if (board[above]==color && labels.notSet(above))
				toDoList.push(above);
			if (board[below]==color && labels.notSet(below))
				toDoList.push(below);
		}
		while (!toDoList.isEmpty());

		return nLibs;
	}

	public static Point getPoint()
	{
		if (pointStack.isEmpty())
			return new Point(0,0,pointStack);

		Point point = (Point) pointStack.pop();
		return point;
	}

	public static Point getPoint(int x, int y)
	{
		if (pointStack.isEmpty())
			return new Point(x,y,pointStack);

		Point point = (Point) pointStack.pop();
		point.x = x;
		point.y = y;
		return point;
	}

	/**
	 	Allocate an UniqueList. Instead of allocating
	 	new objects each time, the objects created through this method
	 	can be 'recycled' and then reused. This is much more efficient
	 	than relying on the VM to allocate an object and then later
	 	garbage-collect it again.
	
	 	<br><br>Creation date: (17-May-01 2:37:36 PM)<br><br>
	*/
	public static UniqueList getUniqueList()
	{
		if (uniqueListStack.isEmpty())
			return new UniqueList(uniqueListStack);

		UniqueList list = (UniqueList) uniqueListStack.pop();
		list.clear();
		return list;
	}

	/**
	 	<br><br>Creation date: (09-May-01 6:25:55 PM)<br><br>
	
	 	@param: int xy is the coordinate of the move.
	 	@param: byte color is the color of the move.
	 	@param: byte[] board is the array containing the stones.
	 	@return: whether a move would make an empty triangle
	*/
	public static boolean isEmptyTriangle(int xy, byte color, byte[] board)
	{
		int left = GoArray.left(xy);
		int right = GoArray.right(xy);
		int above = GoArray.above(xy);
		int below = GoArray.below(xy);
		int left_above = GoArray.left(GoArray.above(xy));
		int left_below = GoArray.left(GoArray.below(xy));
		int right_above = GoArray.right(GoArray.above(xy));
		int right_below = GoArray.right(GoArray.below(xy));
		byte otherColor = GoConst.invertColor(color);
		
		if (board[left]==color && board[above]==color && board[left_above]!=otherColor)
			return true;
		if (board[left]==color && board[below]==color && board[left_below]!=otherColor)
			return true;
		if (board[right]==color && board[above]==color && board[right_above]!=otherColor)
			return true;
		if (board[right]==color && board[below]==color && board[right_below]!=otherColor)
			return true;

		if (board[left]==color)
		{
			if (board[above]==GoConst.EMPTY && board[left_above]==color)
				return true;
			if (board[below]==GoConst.EMPTY && board[left_below]==color)
				return true;
		}
		if (board[right]==color)
		{
			if (board[above]==GoConst.EMPTY && board[right_above]==color)
				return true;
			if (board[below]==GoConst.EMPTY && board[right_below]==color)
				return true;
		}
		if (board[above]==color)
		{
			if (board[left]==GoConst.EMPTY && board[left_above]==color)
				return true;
			if (board[right]==GoConst.EMPTY && board[right_above]==color)
				return true;
		}
		if (board[below]==color)
		{
			if (board[left]==GoConst.EMPTY && board[left_below]==color)
				return true;
			if (board[right]==GoConst.EMPTY && board[right_below]==color)
				return true;
		}

		return false;
	}
}